@@ -87,15 +87,16 @@
 --    transfers control to the following binding-hook lexer:
 --
 --      ident       -> letter (letter | digit | `\'')*
+--      cidenttail  -> digit (letter | digit)*
 --      reservedid  -> `add' | `as' | `call' | `class' | `context' | `deriving'
 --                   | `enum' | `foreign' | `fun' | `get' | `lib'
---                   | `downcaseFirstLetter'
+--                   | `downcaseFirstLetter' | `finalizer'
 --                   | `newtype' | `nocode' | `pointer' | `prefix' | `pure'
 --                   | `set' | `sizeof' | `stable' | `struct' | `type'
 --                   | `underscoreToCase' | `upcaseFirstLetter' | `unsafe' |
---                   | `with'
+--                   | `with' | `const' | `omit'
 --      reservedsym -> `{#' | `#}' | `{' | `}' | `,' | `.' | `->' | `='
---                   | `=>' | '-' | `*' | `&' | `^'
+--                   | `=>' | '-' | `*' | `&' | `^' | `+' | `%'
 --      string      -> `"' instr* `"'
 --      verbhs      -> `\`' inhsverb* `\''
 --      quoths      -> `\'' inhsverb* `\''
@@ -172,7 +173,7 @@ module C2HS.CHS.Lexer (CHSToken(..), lexCHS, keywordToIdent)
 where
 
 import Data.List     ((\\))
-import Data.Char     (isDigit)
+import Data.Char     (isDigit, isSpace)
 
 import Language.C.Data.Ident
 import Language.C.Data.Name
@@ -200,20 +201,31 @@ data CHSToken = CHSTokArrow   Position          -- `->'
               | CHSTokStar    Position          -- `*'
               | CHSTokAmp     Position          -- `&'
               | CHSTokHat     Position          -- `^'
+              | CHSTokPercent Position          -- `%'
+              | CHSTokPlus    Position          -- `+'
+              | CHSTokPlusS   Position          -- `+S'
+              | CHSTokPlusNum Position Int      -- `+<num>'
               | CHSTokLBrace  Position          -- `{'
               | CHSTokRBrace  Position          -- `}'
               | CHSTokLParen  Position          -- `('
               | CHSTokRParen  Position          -- `)'
+              | CHSTokLBrack  Position          -- `['
+              | CHSTokRBrack  Position          -- `]'
               | CHSTokHook    Position          -- `{#'
               | CHSTokEndHook Position          -- `#}'
               | CHSTokAdd     Position          -- `add'
               | CHSTokAs      Position          -- `as'
               | CHSTokCall    Position          -- `call'
               | CHSTokClass   Position          -- `class'
+              | CHSTokConst   Position          -- `const'
               | CHSTokContext Position          -- `context'
+              | CHSTokNonGNU  Position          -- `nonGNU'
+              | CHSTokTypedef Position          -- `typedef'
+              | CHSTokDefault Position          -- `default'
               | CHSTokDerive  Position          -- `deriving'
               | CHSTokDown    Position          -- `downcaseFirstLetter'
               | CHSTokEnum    Position          -- `enum'
+              | CHSTokFinal   Position          -- `finalizer'
               | CHSTokForeign Position          -- `foreign'
               | CHSTokFun     Position          -- `fun'
               | CHSTokGet     Position          -- `get'
@@ -222,6 +234,7 @@ data CHSToken = CHSTokArrow   Position          -- `->'
               | CHSTokNewtype Position          -- `newtype'
               | CHSTokNocode  Position          -- `nocode'
               | CHSTokOffsetof Position         -- `offsetof'
+              | CHSTokOmit    Position          -- `omit'
               | CHSTokPointer Position          -- `pointer'
               | CHSTokPrefix  Position          -- `prefix'
               | CHSTokPure    Position          -- `pure'
@@ -235,7 +248,10 @@ data CHSToken = CHSTokArrow   Position          -- `->'
               | CHSTok_2Case  Position          -- `underscoreToCase'
               | CHSTokUnsafe  Position          -- `unsafe'
               | CHSTokUpper   Position          -- `upcaseFirstLetter'
-              | CHSTokWith    Position          -- `with'
+              | CHSTokVariadic Position          -- `variadic'
+              | CHSTokWith    Position Ident    -- `with'
+              | CHSTokIn      Position          -- `in'
+              | CHSTokOut     Position          -- `out'
               | CHSTokString  Position String   -- string
               | CHSTokHSVerb  Position String   -- verbatim Haskell (`...')
               | CHSTokHSQuot  Position String   -- quoted Haskell ('...')
@@ -245,6 +261,9 @@ data CHSToken = CHSTokArrow   Position          -- `->'
               | CHSTokLine    Position          -- line pragma
               | CHSTokC       Position String   -- verbatim C code
               | CHSTokCtrl    Position Char     -- control code
+              | CHSTokComment Position String   -- comment
+              | CHSTokCIdentTail Position Ident -- C identifier without prefix
+              | CHSTokCArg Position String      -- C type argument
 
 instance Pos CHSToken where
   posOf (CHSTokArrow   pos  ) = pos
@@ -260,15 +279,23 @@ instance Pos CHSToken where
   posOf (CHSTokRBrace  pos  ) = pos
   posOf (CHSTokLParen  pos  ) = pos
   posOf (CHSTokRParen  pos  ) = pos
+  posOf (CHSTokLBrack  pos  ) = pos
+  posOf (CHSTokRBrack  pos  ) = pos
+  posOf (CHSTokHook    pos  ) = pos
   posOf (CHSTokEndHook pos  ) = pos
   posOf (CHSTokAdd     pos  ) = pos
   posOf (CHSTokAs      pos  ) = pos
   posOf (CHSTokCall    pos  ) = pos
   posOf (CHSTokClass   pos  ) = pos
+  posOf (CHSTokConst   pos  ) = pos
   posOf (CHSTokContext pos  ) = pos
+  posOf (CHSTokNonGNU  pos  ) = pos
   posOf (CHSTokDerive  pos  ) = pos
+  posOf (CHSTokTypedef pos  ) = pos
+  posOf (CHSTokDefault pos  ) = pos
   posOf (CHSTokDown    pos  ) = pos
   posOf (CHSTokEnum    pos  ) = pos
+  posOf (CHSTokFinal   pos  ) = pos
   posOf (CHSTokForeign pos  ) = pos
   posOf (CHSTokFun     pos  ) = pos
   posOf (CHSTokGet     pos  ) = pos
@@ -277,6 +304,7 @@ instance Pos CHSToken where
   posOf (CHSTokNewtype pos  ) = pos
   posOf (CHSTokNocode  pos  ) = pos
   posOf (CHSTokOffsetof pos ) = pos
+  posOf (CHSTokOmit    pos  ) = pos
   posOf (CHSTokPointer pos  ) = pos
   posOf (CHSTokPrefix  pos  ) = pos
   posOf (CHSTokPure    pos  ) = pos
@@ -290,7 +318,10 @@ instance Pos CHSToken where
   posOf (CHSTok_2Case  pos  ) = pos
   posOf (CHSTokUnsafe  pos  ) = pos
   posOf (CHSTokUpper   pos  ) = pos
-  posOf (CHSTokWith    pos  ) = pos
+  posOf (CHSTokVariadic pos  ) = pos
+  posOf (CHSTokWith    pos _) = pos
+  posOf (CHSTokIn      pos  ) = pos
+  posOf (CHSTokOut     pos  ) = pos
   posOf (CHSTokString  pos _) = pos
   posOf (CHSTokHSVerb  pos _) = pos
   posOf (CHSTokHSQuot  pos _) = pos
@@ -300,6 +331,10 @@ instance Pos CHSToken where
   posOf (CHSTokLine    pos  ) = pos
   posOf (CHSTokC       pos _) = pos
   posOf (CHSTokCtrl    pos _) = pos
+  posOf (CHSTokComment pos _) = pos
+  posOf (CHSTokCIdentTail pos _) = pos
+  posOf (CHSTokCArg    pos _) = pos
+  posOf (CHSTokPercent pos) = pos
 
 instance Eq CHSToken where
   (CHSTokArrow    _  ) == (CHSTokArrow    _  ) = True
@@ -315,15 +350,23 @@ instance Eq CHSToken where
   (CHSTokRBrace   _  ) == (CHSTokRBrace   _  ) = True
   (CHSTokLParen   _  ) == (CHSTokLParen   _  ) = True
   (CHSTokRParen   _  ) == (CHSTokRParen   _  ) = True
+  (CHSTokLBrack   _  ) == (CHSTokLBrack   _  ) = True
+  (CHSTokRBrack   _  ) == (CHSTokRBrack   _  ) = True
+  (CHSTokHook     _  ) == (CHSTokHook     _  ) = True
   (CHSTokEndHook  _  ) == (CHSTokEndHook  _  ) = True
   (CHSTokAdd      _  ) == (CHSTokAdd      _  ) = True
   (CHSTokAs       _  ) == (CHSTokAs       _  ) = True
   (CHSTokCall     _  ) == (CHSTokCall     _  ) = True
   (CHSTokClass    _  ) == (CHSTokClass    _  ) = True
+  (CHSTokConst    _  ) == (CHSTokConst    _  ) = True
   (CHSTokContext  _  ) == (CHSTokContext  _  ) = True
+  (CHSTokNonGNU   _  ) == (CHSTokNonGNU   _  ) = True
+  (CHSTokTypedef  _  ) == (CHSTokTypedef  _  ) = True
+  (CHSTokDefault  _  ) == (CHSTokDefault  _  ) = True
   (CHSTokDerive   _  ) == (CHSTokDerive   _  ) = True
   (CHSTokDown     _  ) == (CHSTokDown     _  ) = True
   (CHSTokEnum     _  ) == (CHSTokEnum     _  ) = True
+  (CHSTokFinal    _  ) == (CHSTokFinal    _  ) = True
   (CHSTokForeign  _  ) == (CHSTokForeign  _  ) = True
   (CHSTokFun      _  ) == (CHSTokFun      _  ) = True
   (CHSTokGet      _  ) == (CHSTokGet      _  ) = True
@@ -332,6 +375,7 @@ instance Eq CHSToken where
   (CHSTokNewtype  _  ) == (CHSTokNewtype  _  ) = True
   (CHSTokNocode   _  ) == (CHSTokNocode   _  ) = True
   (CHSTokOffsetof _  ) == (CHSTokOffsetof _  ) = True
+  (CHSTokOmit     _  ) == (CHSTokOmit     _  ) = True
   (CHSTokPointer  _  ) == (CHSTokPointer  _  ) = True
   (CHSTokPrefix   _  ) == (CHSTokPrefix   _  ) = True
   (CHSTokPure     _  ) == (CHSTokPure     _  ) = True
@@ -345,7 +389,10 @@ instance Eq CHSToken where
   (CHSTok_2Case   _  ) == (CHSTok_2Case   _  ) = True
   (CHSTokUnsafe   _  ) == (CHSTokUnsafe   _  ) = True
   (CHSTokUpper    _  ) == (CHSTokUpper    _  ) = True
-  (CHSTokWith     _  ) == (CHSTokWith     _  ) = True
+  (CHSTokVariadic _  ) == (CHSTokVariadic _  ) = True
+  (CHSTokWith     _ _) == (CHSTokWith     _ _) = True
+  (CHSTokIn       _  ) == (CHSTokIn       _  ) = True
+  (CHSTokOut      _  ) == (CHSTokOut      _  ) = True
   (CHSTokString   _ _) == (CHSTokString   _ _) = True
   (CHSTokHSVerb   _ _) == (CHSTokHSVerb   _ _) = True
   (CHSTokHSQuot   _ _) == (CHSTokHSQuot   _ _) = True
@@ -355,6 +402,9 @@ instance Eq CHSToken where
   (CHSTokLine     _  ) == (CHSTokLine     _  ) = True
   (CHSTokC        _ _) == (CHSTokC        _ _) = True
   (CHSTokCtrl     _ _) == (CHSTokCtrl     _ _) = True
+  (CHSTokComment  _ _) == (CHSTokComment  _ _) = True
+  (CHSTokCIdentTail _ _) == (CHSTokCIdentTail _ _) = True
+  (CHSTokCArg     _ _) == (CHSTokCArg     _ _) = True
   _                    == _                    = False
 
 instance Show CHSToken where
@@ -367,19 +417,31 @@ instance Show CHSToken where
   showsPrec _ (CHSTokStar    _  ) = showString "*"
   showsPrec _ (CHSTokAmp     _  ) = showString "&"
   showsPrec _ (CHSTokHat     _  ) = showString "^"
+  showsPrec _ (CHSTokPercent _  ) = showString "%"
+  showsPrec _ (CHSTokPlus    _  ) = showString "+"
+  showsPrec _ (CHSTokPlusS   _  ) = showString "+S"
+  showsPrec _ (CHSTokPlusNum _  sz) = showString ("+" ++ show sz)
   showsPrec _ (CHSTokLBrace  _  ) = showString "{"
   showsPrec _ (CHSTokRBrace  _  ) = showString "}"
   showsPrec _ (CHSTokLParen  _  ) = showString "("
   showsPrec _ (CHSTokRParen  _  ) = showString ")"
+  showsPrec _ (CHSTokLBrack  _  ) = showString "["
+  showsPrec _ (CHSTokRBrack  _  ) = showString "]"
+  showsPrec _ (CHSTokHook    _  ) = showString "{#"
   showsPrec _ (CHSTokEndHook _  ) = showString "#}"
   showsPrec _ (CHSTokAdd     _  ) = showString "add"
   showsPrec _ (CHSTokAs      _  ) = showString "as"
   showsPrec _ (CHSTokCall    _  ) = showString "call"
   showsPrec _ (CHSTokClass   _  ) = showString "class"
+  showsPrec _ (CHSTokConst   _  ) = showString "const"
   showsPrec _ (CHSTokContext _  ) = showString "context"
+  showsPrec _ (CHSTokNonGNU  _  ) = showString "nonGNU"
+  showsPrec _ (CHSTokTypedef _  ) = showString "typedef"
+  showsPrec _ (CHSTokDefault _  ) = showString "default"
   showsPrec _ (CHSTokDerive  _  ) = showString "deriving"
   showsPrec _ (CHSTokDown    _  ) = showString "downcaseFirstLetter"
   showsPrec _ (CHSTokEnum    _  ) = showString "enum"
+  showsPrec _ (CHSTokFinal   _  ) = showString "finalizer"
   showsPrec _ (CHSTokForeign _  ) = showString "foreign"
   showsPrec _ (CHSTokFun     _  ) = showString "fun"
   showsPrec _ (CHSTokGet     _  ) = showString "get"
@@ -388,6 +450,7 @@ instance Show CHSToken where
   showsPrec _ (CHSTokNewtype _  ) = showString "newtype"
   showsPrec _ (CHSTokNocode  _  ) = showString "nocode"
   showsPrec _ (CHSTokOffsetof _ ) = showString "offsetof"
+  showsPrec _ (CHSTokOmit    _  ) = showString "omit"
   showsPrec _ (CHSTokPointer _  ) = showString "pointer"
   showsPrec _ (CHSTokPrefix  _  ) = showString "prefix"
   showsPrec _ (CHSTokPure    _  ) = showString "pure"
@@ -401,7 +464,10 @@ instance Show CHSToken where
   showsPrec _ (CHSTok_2Case  _  ) = showString "underscoreToCase"
   showsPrec _ (CHSTokUnsafe  _  ) = showString "unsafe"
   showsPrec _ (CHSTokUpper   _  ) = showString "upcaseFirstLetter"
-  showsPrec _ (CHSTokWith    _  ) = showString "with"
+  showsPrec _ (CHSTokVariadic _  ) = showString "variadic"
+  showsPrec _ (CHSTokWith    _ _) = showString "with"
+  showsPrec _ (CHSTokIn      _  ) = showString "in"
+  showsPrec _ (CHSTokOut     _  ) = showString "out"
   showsPrec _ (CHSTokString  _ s) = showString ("\"" ++ s ++ "\"")
   showsPrec _ (CHSTokHSVerb  _ s) = showString ("`" ++ s ++ "'")
   showsPrec _ (CHSTokHSQuot  _ s) = showString ("'" ++ s ++ "'")
@@ -411,7 +477,11 @@ instance Show CHSToken where
   showsPrec _ (CHSTokLine    _  ) = id            --TODO show line num?
   showsPrec _ (CHSTokC       _ s) = showString s
   showsPrec _ (CHSTokCtrl    _ c) = showChar c
-
+  showsPrec _ (CHSTokComment _ s) = showString (if null s
+                                                then ""
+                                                else " --" ++ s ++ "\n")
+  showsPrec _ (CHSTokCIdentTail _ i) = (showString . identToString) i
+  showsPrec _ (CHSTokCArg    _ s) = showString s
 
 -- lexer state
 -- -----------
@@ -609,7 +679,7 @@ hook  = string "{#"
 --
 startmarker :: CHSLexer
 startmarker = char '\000' `lexmeta`
-              \lexeme pos s -> (Nothing, incPos pos 1, s, Just chslexer)
+              \_ pos s -> (Nothing, incPos pos 1, s, Just chslexer)
 
 -- | pre-processor directives and `#c'
 --
@@ -621,11 +691,13 @@ cpp :: CHSLexer
 cpp = directive
       where
         directive =
-          (string "\n#" >|< string "\0#") +>
+          --(string "\n#" >|< string "\0#") +>
+          alt "\n\0" +> alt " \t" `star` string "#" +>
           alt ('\t':inlineSet)`star` epsilon
           `lexmeta`
-             \t@(ld:_:dir) pos s ->      -- strip off the "\n#" or "\0#"
-               case dir of
+             \t@(ld:spdir) pos s ->      -- strip off the "\n" or "\0"
+             let dir = drop 1 $ dropWhile (`elem` " \t") spdir
+             in case dir of
                  ['c']                      ->          -- #c
                    (Nothing, incPos pos (length t), s, Just cLexer)
                  -- a #c may be followed by whitespace
@@ -664,15 +736,34 @@ bhLexer  =      identOrKW
            >||< hsverb
            >||< hsquot
            >||< whitespace
+           >||< arglist
            >||< endOfHook
-           >||< string "--" +> anyButNL`star` char '\n'   -- comment
-                `lexmeta` \_ pos s -> (Nothing, retPos pos, s, Nothing)
+           >||< string "--" +> anyButNL`star` epsilon  -- comment
+                `lexaction` \cs pos -> Just (CHSTokComment pos (drop 2 cs))
            where
              anyButNL  = alt (anySet \\ ['\n'])
              endOfHook = string "#}"
                          `lexmeta`
                           \_ pos s -> (Just $ Right (CHSTokEndHook pos),
                                        incPos pos 2, s, Just chslexer)
+             arglist = string "["
+                       `lexmeta` \_ pos s -> (Just $ Right (CHSTokLBrack pos),
+                                              incPos pos 1, s, Just alLexer)
+
+-- | lexer for C function types for variadic functions
+--
+alLexer :: CHSLexer
+alLexer =      sym ","  CHSTokComma
+          >||< endOfArgList
+          >||< cArg
+  where sym cs con = string cs `lexaction` \_ pos -> Just (con pos)
+        endOfArgList = string "]"
+                       `lexmeta`
+                       \_ pos s -> (Just $ Right (CHSTokRBrack pos),
+                                    incPos pos 1, s, Just bhLexer)
+        cArg = ((alt (anySet \\ ",]")) `star` epsilon) `lexaction`
+               \cs pos -> Just (CHSTokCArg pos $ trim cs)
+        trim = reverse . dropWhile isSpace . reverse . dropWhile isSpace
 
 -- | the inline-C lexer
 --
@@ -708,15 +799,24 @@ identOrKW  =
        -- identifier or keyword
        (letter +> (letter >|< digit >|< char '\'')`star` epsilon
        `lexactionName` \cs pos name -> (idkwtok $!pos) cs name)
+       >||<
+       (digit +> (letter >|< digit)`star` epsilon
+        `lexactionName` \cs pos name ->
+        CHSTokCIdentTail pos (mkIdent pos cs name))
   where
     idkwtok pos "add"              _    = CHSTokAdd     pos
     idkwtok pos "as"               _    = CHSTokAs      pos
     idkwtok pos "call"             _    = CHSTokCall    pos
     idkwtok pos "class"            _    = CHSTokClass   pos
+    idkwtok pos "const"            _    = CHSTokConst   pos
     idkwtok pos "context"          _    = CHSTokContext pos
+    idkwtok pos "nonGNU"           _    = CHSTokNonGNU  pos
+    idkwtok pos "typedef"          _    = CHSTokTypedef pos
+    idkwtok pos "default"          _    = CHSTokDefault pos
     idkwtok pos "deriving"         _    = CHSTokDerive  pos
     idkwtok pos "downcaseFirstLetter" _ = CHSTokDown    pos
     idkwtok pos "enum"             _    = CHSTokEnum    pos
+    idkwtok pos "finalizer"        _    = CHSTokFinal   pos
     idkwtok pos "foreign"          _    = CHSTokForeign pos
     idkwtok pos "fun"              _    = CHSTokFun     pos
     idkwtok pos "get"              _    = CHSTokGet     pos
@@ -725,6 +825,7 @@ identOrKW  =
     idkwtok pos "newtype"          _    = CHSTokNewtype pos
     idkwtok pos "nocode"           _    = CHSTokNocode  pos
     idkwtok pos "offsetof"         _    = CHSTokOffsetof pos
+    idkwtok pos "omit"             _    = CHSTokOmit    pos
     idkwtok pos "pointer"          _    = CHSTokPointer pos
     idkwtok pos "prefix"           _    = CHSTokPrefix  pos
     idkwtok pos "pure"             _    = CHSTokPure    pos
@@ -738,10 +839,14 @@ identOrKW  =
     idkwtok pos "underscoreToCase" _    = CHSTok_2Case  pos
     idkwtok pos "unsafe"           _    = CHSTokUnsafe  pos
     idkwtok pos "upcaseFirstLetter"_    = CHSTokUpper   pos
-    idkwtok pos "with"             _    = CHSTokWith    pos
+    idkwtok pos "variadic"         _    = CHSTokVariadic pos
+    idkwtok pos "with"             name = mkwith pos name
+    idkwtok pos "in"               _    = CHSTokIn      pos
+    idkwtok pos "out"              _    = CHSTokOut     pos
     idkwtok pos cs                 name = mkid pos cs name
     --
     mkid pos cs name = CHSTokIdent pos (mkIdent pos cs name)
+    mkwith pos name = CHSTokWith pos (mkIdent pos "with" name)
 
 keywordToIdent :: CHSToken -> CHSToken
 keywordToIdent tok =
@@ -750,10 +855,15 @@ keywordToIdent tok =
     CHSTokAs      pos -> mkid pos "as"
     CHSTokCall    pos -> mkid pos "call"
     CHSTokClass   pos -> mkid pos "class"
+    CHSTokConst   pos -> mkid pos "const"
     CHSTokContext pos -> mkid pos "context"
+    CHSTokNonGNU  pos -> mkid pos "nonGNU"
+    CHSTokTypedef pos -> mkid pos "typedef"
+    CHSTokDefault pos -> mkid pos "default"
     CHSTokDerive  pos -> mkid pos "deriving"
     CHSTokDown    pos -> mkid pos "downcaseFirstLetter"
     CHSTokEnum    pos -> mkid pos "enum"
+    CHSTokFinal   pos -> mkid pos "finalizer"
     CHSTokForeign pos -> mkid pos "foreign"
     CHSTokFun     pos -> mkid pos "fun"
     CHSTokGet     pos -> mkid pos "get"
@@ -762,6 +872,7 @@ keywordToIdent tok =
     CHSTokNewtype pos -> mkid pos "newtype"
     CHSTokNocode  pos -> mkid pos "nocode"
     CHSTokOffsetof pos -> mkid pos "offsetof"
+    CHSTokOmit    pos -> mkid pos "omit"
     CHSTokPointer pos -> mkid pos "pointer"
     CHSTokPrefix  pos -> mkid pos "prefix"
     CHSTokPure    pos -> mkid pos "pure"
@@ -775,7 +886,10 @@ keywordToIdent tok =
     CHSTok_2Case  pos -> mkid pos "underscoreToCase"
     CHSTokUnsafe  pos -> mkid pos "unsafe"
     CHSTokUpper   pos -> mkid pos "upcaseFirstLetter"
-    CHSTokWith    pos -> mkid pos "with"
+    CHSTokVariadic pos -> mkid pos "variadic"
+    CHSTokWith    pos ide -> CHSTokIdent pos ide
+    CHSTokIn      pos -> mkid pos "in"
+    CHSTokOut     pos -> mkid pos "out"
     _ -> tok
     where mkid pos str = CHSTokIdent pos (internalIdent str)
 
@@ -791,12 +905,19 @@ symbol  =      sym "->" CHSTokArrow
           >||< sym "*"  CHSTokStar
           >||< sym "&"  CHSTokAmp
           >||< sym "^"  CHSTokHat
+          >||< sym "%"  CHSTokPercent
+          >||< sym "+"  CHSTokPlus
+          >||< sym "+S" CHSTokPlusS
+          >||< sym_with_num "+" CHSTokPlusNum
           >||< sym "{"  CHSTokLBrace
           >||< sym "}"  CHSTokRBrace
           >||< sym "("  CHSTokLParen
           >||< sym ")"  CHSTokRParen
           where
             sym cs con = string cs `lexaction` \_ pos -> Just (con pos)
+            sym_with_num cs con =
+              string cs +> digit +> digit`star` epsilon
+              `lexaction` \(_:ds) pos -> Just (con pos (read ds))
 
 -- | string
 --
